// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
import { IRouter } from '@jupyterlab/application';
import { IConsoleTracker } from '@jupyterlab/console';
import { PageConfig, PathExt } from '@jupyterlab/coreutils';
import { IDocumentManager } from '@jupyterlab/docmanager';
import { liteWordmark } from '@jupyterlite/ui-components';
import { Widget } from '@lumino/widgets';
/**
 * The default notebook factory.
 */
const NOTEBOOK_FACTORY = 'Notebook';
/**
 * The editor factory.
 */
const EDITOR_FACTORY = 'Editor';
/**
 * A regular expression to match path to notebooks, documents and consoles
 */
const URL_PATTERN = new RegExp('/(notebooks|edit|consoles)\\/?');
/**
 * Open consoles in a new tab.
 */
const consoles = {
    id: '@jupyterlite/retro-application-extension:consoles',
    requires: [IConsoleTracker],
    autoStart: true,
    activate: (app, tracker) => {
        const baseUrl = PageConfig.getBaseUrl();
        tracker.widgetAdded.connect(async (send, console) => {
            const { sessionContext } = console;
            const page = PageConfig.getOption('retroPage');
            if (page === 'consoles') {
                return;
            }
            const path = sessionContext.path;
            window.open(`${baseUrl}retro/consoles?path=${path}`, '_blank');
            // the widget is not needed anymore
            console.dispose();
        });
    }
};
/**
 * A plugin to open document in a new browser tab.
 *
 * TODO: remove and use a custom doc manager?
 */
const docmanager = {
    id: '@jupyterlite/retro-application-extension:docmanager',
    requires: [IDocumentManager],
    autoStart: true,
    activate: (app, docManager) => {
        const baseUrl = PageConfig.getBaseUrl();
        // patch the `docManager.open` option to prevent the default behavior
        const docOpen = docManager.open;
        docManager.open = (path, widgetName = 'default', kernel, options) => {
            const ref = options === null || options === void 0 ? void 0 : options.ref;
            if (ref === '_noref') {
                docOpen.call(docManager, path, widgetName, kernel, options);
                return;
            }
            const ext = PathExt.extname(path);
            const route = ext === '.ipynb' ? 'notebooks' : 'edit';
            window.open(`${baseUrl}retro/${route}?path=${path}`);
            return undefined;
        };
    }
};
/**
 * The logo plugin.
 */
const logo = {
    id: '@jupyterlite/retro-application-extension:logo',
    autoStart: true,
    activate: (app) => {
        const baseUrl = PageConfig.getBaseUrl();
        const node = document.createElement('a');
        node.href = `${baseUrl}retro/tree`;
        node.target = '_blank';
        node.rel = 'noopener noreferrer';
        const logo = new Widget({ node });
        liteWordmark.element({
            container: node,
            elementPosition: 'center',
            padding: '2px 2px 2px 8px',
            height: '28px',
            width: 'auto'
        });
        logo.id = 'jp-RetroLogo';
        app.shell.add(logo, 'top', { rank: 0 });
    }
};
/**
 * A custom opener plugin to pass the path to documents as
 * query string parameters.
 */
const opener = {
    id: '@jupyterlite/retro-application-extension:opener',
    autoStart: true,
    requires: [IRouter, IDocumentManager],
    activate: (app, router, docManager) => {
        const { commands } = app;
        const command = 'router:tree';
        commands.addCommand(command, {
            execute: (args) => {
                var _a;
                const parsed = args;
                // use request to do the matching
                const matches = (_a = parsed.request.match(URL_PATTERN)) !== null && _a !== void 0 ? _a : [];
                if (!matches) {
                    return;
                }
                const urlParams = new URLSearchParams(window.location.search);
                const path = urlParams.get('path');
                if (!path) {
                    return;
                }
                const file = decodeURIComponent(path);
                app.restored.then(() => {
                    const page = PageConfig.getOption('retroPage');
                    switch (page) {
                        case 'consoles': {
                            commands.execute('console:create', { path: file });
                            return;
                        }
                        case 'notebooks': {
                            docManager.open(file, NOTEBOOK_FACTORY, undefined, {
                                ref: '_noref'
                            });
                            return;
                        }
                        case 'edit': {
                            docManager.open(file, EDITOR_FACTORY, undefined, {
                                ref: '_noref'
                            });
                            return;
                        }
                    }
                });
            }
        });
        router.register({ command, pattern: URL_PATTERN });
    }
};
const plugins = [consoles, docmanager, logo, opener];
export default plugins;
